package com.tzld.longarticle.recommend.server.service.score.strategy;

import cn.hutool.core.io.resource.ResourceUtil;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.service.score.*;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.NormalizationUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Component
public class ViewMultiplierStrategy implements ScoreStrategy {

    @Autowired
    NamedParameterJdbcTemplate programDataJdbcTemplate;
    private JSONObject jsonObject;

    @PostConstruct
    public void init() {
        String cardJSON = ResourceUtil.readUtf8Str("file/AccountInfo.json");
        jsonObject = JSONObject.parseObject(cardJSON);
    }

    @Override
    public List<Score> score(ScoreParam param) {

        if (CollectionUtils.isEmpty(param.getContents())) {
            return Collections.emptyList();
        }
        List<String> channelContentIds = param.getContents().stream().map(Content::getCrawlerChannelContentId).collect(Collectors.toList());
        Map<String, ChangwenArticleDTO> changwenArticleDTOMap = getContentIndex(channelContentIds);
        Map<String, ChangwenArticleDatastatDTO> changwenArticleDatastatDTOMap = getContentViewCount(channelContentIds);

        List<Score> scores = CommonCollectionUtils.toList(param.getContents(), c -> {
            Score score = new Score();
            score.setContentId(c.getId());
            ChangwenArticleDTO articleDTO = changwenArticleDTOMap.get(c.getCrawlerChannelContentId());
            ChangwenArticleDatastatDTO datastatDTO = changwenArticleDatastatDTOMap.get(c.getCrawlerChannelContentId());

            if (Objects.nonNull(articleDTO) && Objects.nonNull(datastatDTO)) {
                double avgReadCount = getAvgReadCount(articleDTO.getAccountId(), articleDTO.getItemIndex());
                Integer readCount = datastatDTO.getReadCount();
                score.setScore(NormalizationUtils.min(readCount / avgReadCount - 1));
            } else {
                score.setScore(-1.0);
            }
            score.setStrategy(this);
            return score;
        });
        return scores;
    }

    private Map<String, ChangwenArticleDTO> getContentIndex(List<String> channelContentIds) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("channelContentIds", channelContentIds);
        List<ChangwenArticleDTO> result = programDataJdbcTemplate.query(
                "select id, account_id, item_index from changwen_article where id in (:channelContentIds) ",
                parameters,
                new BeanPropertyRowMapper<>(ChangwenArticleDTO.class));
        if (CollectionUtils.isEmpty(result)) {
            return new HashMap<>();
        }
        Map<String, ChangwenArticleDTO> map = result.stream().collect(Collectors.toMap(ChangwenArticleDTO::getId, o -> o));
        return map;
    }

    private Map<String, ChangwenArticleDatastatDTO> getContentViewCount(List<String> channelContentIds) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("channelContentIds", channelContentIds);
        List<ChangwenArticleDatastatDTO> result = programDataJdbcTemplate.query(
                "select article_id, read_count from changwen_article_datastat where article_id in (:channelContentIds) ",
                parameters,
                new BeanPropertyRowMapper<>(ChangwenArticleDatastatDTO.class));
        if (CollectionUtils.isEmpty(result)) {
            return new HashMap<>();
        }
        Map<String, ChangwenArticleDatastatDTO> map = result.stream().collect(Collectors.toMap(ChangwenArticleDatastatDTO::getArticleId, o -> o));
        return map;
    }

    private double getAvgReadCount(String ghId, Integer index) {
        AvgReadDTO dto = jsonObject.getObject(ghId + "_" + index, AvgReadDTO.class);
        return dto == null ? 1.0 : dto.getReadAvg();
    }
}
